<?php

namespace NoccyLabs\React\WebSocket\Group;

use Evenement\EventEmitterInterface;
use Evenement\EventEmitterTrait;
use NoccyLabs\React\WebSocket\WebSocketInterface;
use React\EventLoop\Loop;
use WeakReference;

class GroupManager implements EventEmitterInterface
{
    use EventEmitterTrait;

    const EVENT_CREATED = 'created';
    const EVENT_DESTROYED = 'destroyed';

    /** @var array<string,ConnectionGroup> */
    private array $groups = [];

    public function getConnectionGroup(string $name): ConnectionGroup
    {
        if (!array_key_exists($name, $this->groups)) {
            $group = new ConnectionGroup($name);
            $this->groups[$name] = $group;
            // Clean up group when the last member leaves
            $group->on(ConnectionGroup::EVENT_LEAVE, function () use ($group) {
                Loop::futureTick(function () use ($group) {
                    if (count($group) === 0) {
                        $this->emit(self::EVENT_DESTROYED, [ $group ]);
                        $group->removeAllListeners();
                        unset($this->groups[$group->getName()]);
                    }
                });
            });
            $this->emit(self::EVENT_CREATED, [ $group ]);
        } else {
            $group = $this->groups[$name];
        }
        return $group;
    }

    
}